@@ -20,6 +20,7 @@ from pathlib import Path
 import pkgutil
 import traceback
 import warnings
+import numbers
 
 import numpy as np
 from qtpy import QtCore
@@ -137,7 +138,7 @@ def set_qt_backend():
         logger.warning(f"The chosen Qt backend ({config('qtbackend', 'backend')}) has not been installed...\n"
                        f"Trying another...")
         backends = config('qtbackend', 'backends')
-        backends.pop(config('qtbackend', 'backend'))
+        backends.pop(backends.index(config('qtbackend', 'backend')))
         for backend in backends:
             if backend.lower() in [mod.lower() for mod in sys.modules]:
                 backend_present = True
@@ -149,7 +150,10 @@ def set_qt_backend():
         logger.info(f"{config('qtbackend', 'backend')} Qt backend loaded")
         logger.info('************************')
     else:
-        logger.critical(f"No Qt backend could be found in your system, plese install either pyqt5/6 or pyside2/6")
+        msg = f"No Qt backend could be found in your system, please install either pyqt5/6 or pyside2/6." \
+              f"pyqt5 is still preferred, while pyqt6 should mostly work."
+        logger.critical(msg)
+        print(msg.upper())
 
 
 class JsonConverter:
@@ -576,6 +580,10 @@ class Axis(AxisBase):
             raise TypeError('data for the Axis class should be a ndarray')
         self.update(kwargs)
 
+    def __mul__(self, other):
+        if isinstance(other, numbers.Number):
+            return Axis(data=self['data'] * other, label=self['label'], units=self['units'])
+
 
 class NavAxis(Axis):
     def __init__(self, data=None, label='', units='', nav_index=-1, **kwargs):
@@ -729,7 +737,7 @@ class DataToExport(Data):
         source: (str) either 'raw' for raw data or 'roi' for data extracted from a roi
         """
         super().__init__(source=source, **kwargs)
-        if data is None or isinstance(data, np.ndarray) or isinstance(data, float) or isinstance(data, int):
+        if data is None or isinstance(data, np.ndarray) or isinstance(data, numbers.Number):
             self['data'] = data
         else:
             raise TypeError('data for the DataToExport class should be a scalar or a ndarray')
@@ -757,6 +765,25 @@ class DataToExport(Data):
                f' - <source: {self.source}> - <dim: {self.dim}>'
 
 
+def ensure_ndarray(data):
+    """
+    Make sure data is returned as a numpy array
+    Parameters
+    ----------
+    data
+
+    Returns
+    -------
+    ndarray
+    """
+    if not isinstance(data, np.ndarray):
+        if isinstance(data, list):
+            data = np.array(data)
+        else:
+            data = np.array([data])
+    return data
+
+
 def setLocale():
     """
     defines the Locale to use to convert numbers to strings representation using language/country conventions
@@ -1250,407 +1277,86 @@ def get_new_file_name(base_path=Path(config('data_saving', 'h5file', 'save_path'
 
 # ##############
 # Math utilities
+
+# math utility functions, should now be imported from the math_utils module
+import pymodaq.daq_utils.math_utils as mutils
+
 def my_moment(x, y):
-    """Returns the moments of a distribution y over an axe x
-
-    Parameters
-    ----------
-    x: list or ndarray
-       vector of floats
-    y: list or ndarray
-       vector of floats corresponding to the x axis
-
-    Returns
-    -------
-    m: list
-       Contains moment of order 0 (mean) and of order 1 (std) of the distribution y
-    """
-    dx = np.mean(np.diff(x))
-    norm = np.sum(y) * dx
-    m = [np.sum(x * y) * dx / norm]
-    m.extend([np.sqrt(np.sum((x - m[0]) ** 2 * y) * dx / norm)])
-    return m
-
+    deprecation_msg(f'my_moment function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.my_moment(x, y)
 
 def normalize(x):
-    x = x - np.min(x)
-    x = x / np.max(x)
-    return x
+    deprecation_msg(f'normalize function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.normalize(x)
 
 
 def odd_even(x):
-    """
-    odd_even tells if a number is odd (return True) or even (return False)
-
-    Parameters
-    ----------
-    x: the integer number to test
-
-    Returns
-    -------
-    bool : boolean
-    """
-    if not isinstance(x, int):
-        raise TypeError(f'{x} should be an integer')
-    if int(x) % 2 == 0:
-        bool = False
-    else:
-        bool = True
-    return bool
+    deprecation_msg(f'odd_even function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.odd_even(x)
 
 
 def greater2n(x):
-    """
-    return the first power of 2 greater than x
-    Parameters
-    ----------
-    x: (int or float) a number
-
-    Returns
-    -------
-    int: the power of 2 greater than x
-    """
-    if isinstance(x, bool):
-        raise TypeError(f'{x} should be an integer or a float')
-    if hasattr(x, '__iter__'):
-        res = []
-        for el in x:
-            if isinstance(el, bool):
-                raise TypeError(f'{el} should be an integer or a float')
-            if not (isinstance(el, int) or isinstance(el, float)):
-                raise TypeError(f'{x} elements should be integer or float')
-            res.append(1 << (int(el) - 1).bit_length())
-        if isinstance(x, np.ndarray):
-            return np.array(res)
-        else:
-            return res
-    else:
-        if not (isinstance(x, int) or isinstance(x, float)):
-            raise TypeError(f'{x} should be an integer or a float')
-        return 1 << (int(x) - 1).bit_length()
+    deprecation_msg(f'greater2n function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.greater2n(x)
 
 
 def linspace_step(start, stop, step):
-    """
-    Compute a regular linspace_step distribution from start to stop values.
-
-    =============== =========== ======================================
-    **Parameters**    **Type**    **Description**
-    *start*            scalar      the starting value of distribution
-    *stop*             scalar      the stopping value of distribution
-    *step*             scalar      the length of a distribution step
-    =============== =========== ======================================
-
-    Returns
-    -------
-
-    scalar array
-        The computed distribution axis as an array.
-    """
-    if np.abs(step) < 1e-12 or np.sign(stop - start) != np.sign(step) or start == stop:
-        raise ValueError('Invalid value for one parameter')
-    Nsteps = int(np.ceil((stop - start) / step))
-    new_stop = start + (Nsteps - 1) * step
-    if np.abs(new_stop + step - stop) < 1e-12:
-        Nsteps += 1
-    new_stop = start + (Nsteps - 1) * step
-    return np.linspace(start, new_stop, Nsteps)
+    deprecation_msg(f'linspace_step function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.linspace_step(start, stop, step)
 
 
 def linspace_step_N(start, step, Npts):
-    stop = (Npts - 1) * step + start
-    return linspace_step(start, stop, step)
+    deprecation_msg(f'linspace_step_N function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.linspace_step_N(start, step, Npts)
 
 
 def find_index(x, threshold):
-    """
-    find_index finds the index ix such that x(ix) is the closest from threshold
-    Parameters
-    ----------
-    x : vector
-    threshold : list of scalar
-
-    Returns
-    -------
-    out : list of 2-tuple containing ix,x[ix]
-            out=[(ix0,xval0),(ix1,xval1),...]
-    """
-
-    if not hasattr(threshold, '__iter__'):
-        threshold = [threshold]
-    out = []
-    for value in threshold:
-        ix = int(np.argmin(np.abs(x - value)))
-        out.append((ix, x[ix]))
-    return out
+    deprecation_msg(f'find_index function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.find_index(x, threshold)
 
 
 def find_common_index(x, y, x0, y0):
-    vals = x + 1j * y
-    val = x0 + 1j * y0
-    ind = int(np.argmin(np.abs(vals - val)))
-    return ind, x[ind], y[ind]
+    deprecation_msg(f'find_common_index function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.find_common_index(x, y, x0, y0)
 
 
 def gauss1D(x, x0, dx, n=1):
-    """
-    compute the gaussian function along a vector x, centered in x0 and with a
-    FWHM i intensity of dx. n=1 is for the standart gaussian while n>1 defines
-    a hypergaussian
-
-    Parameters
-    ----------
-    x: (ndarray) first axis of the 2D gaussian
-    x0: (float) the central position of the gaussian
-    dx: (float) :the FWHM of the gaussian
-    n=1 : an integer to define hypergaussian, n=1 by default for regular gaussian
-    Returns
-    -------
-    out : vector
-      the value taken by the gaussian along x axis
-
-    """
-    if dx <= 0:
-        raise ValueError('dx should be strictly positive')
-    if not isinstance(n, int):
-        raise TypeError('n should be a positive integer')
-    elif n < 0:
-        raise ValueError('n should be a positive integer')
-    out = np.exp(-2 * np.log(2) ** (1 / n) * (((x - x0) / dx)) ** (2 * n))
-    return out
-
-
-# def rotate_2D_array(arr, angle):
-#     theta = np.radians(angle)
-#     c, s = np.cos(theta), np.sin(theta)
-#     R = np.array(((c, -s), (s, c)))
-#     (x0r, y0r) = tuple(R.dot(np.array([x0, y0])))
-#
-#     data = np.zeros((len(y), len(x)))
-#
-#     for indx, xtmp in enumerate(x):
-#         for indy, ytmp in enumerate(y):
-#             rotatedvect = R.dot(np.array([xtmp, ytmp]))
-#             data[indy, indx] = np.exp(
-#                 -2 * np.log(2) ** (1 / n) * ((rotatedvect[0] - x0r) / dx) ** (2 * n)) * np.exp(
-#                 -2 * np.log(2) ** (1 / n) * ((rotatedvect[1] - y0r) / dy) ** (2 * n))
-#
-#     return data
+    deprecation_msg(f'gauss1D function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.gauss1D(x, x0, dx, n=n)
 
 
 def gauss2D(x, x0, dx, y, y0, dy, n=1, angle=0):
-    """
-    compute the 2D gaussian function along a vector x, centered in x0 and with a
-    FWHM in intensity of dx and smae along y axis. n=1 is for the standard gaussian while n>1 defines
-    a hypergaussian. optionally rotate it by an angle in degree
-
-    Parameters
-    ----------
-    x: (ndarray) first axis of the 2D gaussian
-    x0: (float) the central position of the gaussian
-    dx: (float) :the FWHM of the gaussian
-    y: (ndarray) second axis of the 2D gaussian
-    y0: (float) the central position of the gaussian
-    dy: (float) :the FWHM of the gaussian
-    n=1 : an integer to define hypergaussian, n=1 by default for regular gaussian
-    angle: (float) a float to rotate main axes, in degree
-
-    Returns
-    -------
-    out : ndarray 2 dimensions
-
-    """
-    if angle == 0:
-        data = np.transpose(np.outer(gauss1D(x, x0, dx, n), gauss1D(y, y0, dy, n)))
-
-    else:
-
-        theta = np.radians(angle)
-        c, s = np.cos(theta), np.sin(theta)
-        R = np.array(((c, -s), (s, c)))
-        (x0r, y0r) = tuple(R.dot(np.array([x0, y0])))
-
-        data = np.zeros((len(y), len(x)))
-
-        for indx, xtmp in enumerate(x):
-            for indy, ytmp in enumerate(y):
-                rotatedvect = R.dot(np.array([xtmp, ytmp]))
-                data[indy, indx] = gauss1D(rotatedvect[0], x0r, dx, n) * gauss1D(rotatedvect[1], y0r, dy, n)
-
-    return data
-
+    deprecation_msg(f'gauss2D function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.gauss2D(x, x0, dx, y, y0, dy, n, angle)
 
 def ftAxis(Npts, omega_max):
-    """
-    Given two numbers Npts,omega_max, return two vectors spanning the temporal
-    and spectral range. They are related by Fourier Transform
-
-    Parameters
-    ----------
-    Npts: (int)
-      A number of points defining the length of both grids
-    omega_max: (float)
-      The maximum circular frequency in the spectral domain. its unit defines
-      the temporal units. ex: omega_max in rad/fs implies time_grid in fs
-
-    Returns
-    -------
-    omega_grid: (ndarray)
-      The spectral axis of the FFT
-    time_grid: (ndarray))
-      The temporal axis of the FFT
-    See Also
-    --------
-    ftAxis, ftAxis_time, ift, ft2, ift2
-    """
-    if not isinstance(Npts, int):
-        raise TypeError('n should be a positive integer, if possible power of 2')
-    elif Npts < 1:
-        raise ValueError('n should be a strictly positive integer')
-    dT = 2 * np.pi / (2 * omega_max)
-    omega_grid = np.linspace(-omega_max, omega_max, Npts)
-    time_grid = dT * np.linspace(-(Npts - 1) / 2, (Npts - 1) / 2, Npts)
-    return omega_grid, time_grid
+    deprecation_msg(f'ftAxis function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.ftAxis(Npts, omega_max)
 
 
 def ftAxis_time(Npts, time_max):
-    """
-    Given two numbers Npts,omega_max, return two vectors spanning the temporal
-    and spectral range. They are related by Fourier Transform
-
-    Parameters
-    ----------
-    Npts : number
-      A number of points defining the length of both grids
-    time_max : number
-      The maximum tmporal window
-
-    Returns
-    -------
-    omega_grid : vector
-      The spectral axis of the FFT
-    time_grid : vector
-      The temporal axis of the FFT
-    See Also
-    --------
-    ftAxis, ftAxis_time, ift, ft2, ift2
-    """
-    if not isinstance(Npts, int):
-        raise TypeError('n should be a positive integer, if possible power of 2')
-    elif Npts < 1:
-        raise ValueError('n should be a strictly positive integer')
-    dT = time_max / Npts
-    omega_max = (Npts - 1) / 2 * 2 * np.pi / time_max
-    omega_grid = np.linspace(-omega_max, omega_max, Npts)
-    time_grid = dT * np.linspace(-(Npts - 1) / 2, (Npts - 1) / 2, Npts)
-    return omega_grid, time_grid
+    deprecation_msg(f'ftAxis_time function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.ftAxis_time(Npts, time_max)
 
 
 def ft(x, dim=-1):
-    """
-    Process the 1D fast fourier transform and swaps the axis to get coorect results using ftAxis
-    Parameters
-    ----------
-    x: (ndarray) the array on which the FFT should be done
-    dim: the axis over which is done the FFT (default is the last of the array)
-
-    Returns
-    -------
-    See Also
-    --------
-    ftAxis, ftAxis_time, ift, ft2, ift2
-    """
-    if not isinstance(dim, int):
-        raise TypeError('dim should be an integer specifying the array dimension over which to do the calculation')
-    assert isinstance(x, np.ndarray)
-    assert dim >= -1
-    assert dim <= len(x.shape) - 1
-
-    out = np.fft.fftshift(np.fft.fft(np.fft.fftshift(x, axes=dim), axis=dim), axes=dim)
-    return out
+    deprecation_msg(f'ft function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.ft(x, dim)
 
 
 def ift(x, dim=0):
-    """
-    Process the inverse 1D fast fourier transform and swaps the axis to get correct results using ftAxis
-    Parameters
-    ----------
-    x: (ndarray) the array on which the FFT should be done
-    dim: the axis over which is done the FFT (default is the last of the array)
-
-    Returns
-    -------
-    See Also
-    --------
-    ftAxis, ftAxis_time, ift, ft2, ift2
-    """
-    if not isinstance(dim, int):
-        raise TypeError('dim should be an integer specifying the array dimension over which to do the calculation')
-    assert isinstance(x, np.ndarray)
-    assert dim >= -1
-    assert dim <= len(x.shape) - 1
-    out = np.fft.fftshift(np.fft.ifft(np.fft.fftshift(x, axes=dim), axis=dim), axes=dim)
-    return out
+    deprecation_msg(f'ift function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.ift(x, dim)
 
 
 def ft2(x, dim=(-2, -1)):
-    """
-    Process the 2D fast fourier transform and swaps the axis to get correct results using ftAxis
-    Parameters
-    ----------
-    x: (ndarray) the array on which the FFT should be done
-    dim: the axis over which is done the FFT (default is the last of the array)
-
-    Returns
-    -------
-    See Also
-    --------
-    ftAxis, ftAxis_time, ift, ft2, ift2
-    """
-    assert isinstance(x, np.ndarray)
-    if hasattr(dim, '__iter__'):
-        for d in dim:
-            if not isinstance(d, int):
-                raise TypeError(
-                    'elements in dim should be an integer specifying the array dimension over which to do the calculation')
-            assert d <= len(x.shape)
-    else:
-        if not isinstance(dim, int):
-            raise TypeError(
-                'elements in dim should be an integer specifying the array dimension over which to do the calculation')
-        assert dim <= len(x.shape)
-    out = np.fft.fftshift(np.fft.fft2(np.fft.fftshift(x, axes=dim)), axes=dim)
-    return out
+    deprecation_msg(f'ft2 function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.ft2(x, dim)
 
 
 def ift2(x, dim=(-2, -1)):
-    """
-    Process the inverse 2D fast fourier transform and swaps the axis to get correct results using ftAxis
-    Parameters
-    ----------
-    x: (ndarray) the array on which the FFT should be done
-    dim: the axis (or a tuple of axes) over which is done the FFT (default is the last of the array)
-
-    Returns
-    -------
-    See Also
-    --------
-    ftAxis, ftAxis_time, ift, ft2, ift2
-    """
-    assert isinstance(x, np.ndarray)
-    if hasattr(dim, '__iter__'):
-        for d in dim:
-            if not isinstance(d, int):
-                raise TypeError(
-                    'elements in dim should be an integer specifying the array dimension over which to do the calculation')
-            assert d <= len(x.shape)
-    else:
-        if not isinstance(dim, int):
-            raise TypeError(
-                'elements in dim should be an integer specifying the array dimension over which to do the calculation')
-        assert dim <= len(x.shape)
-    out = np.fft.fftshift(np.fft.ifft2(np.fft.fftshift(x, axes=dim)), axes=dim)
-    return out
+    deprecation_msg(f'ift2 function should now be imported from the {mutils.__name__} module', stacklevel=3)
+    return mutils.ift2(x, dim)
 
 
 if __name__ == '__main__':
@@ -1668,12 +1374,12 @@ if __name__ == '__main__':
     # import license
     # mit = license.find('MIT')
     #
-    paths = recursive_find_expr_in_files('C:\\Users\\weber\\Labo\\Programmes Python\\PyMoDAQ_Git',
-                                         exp='DAQmxTypes',
-                                         paths=[],
-                                         filters=['.git', '.idea', '__pycache__', 'build', 'egg', 'documentation',
-                                                  '.tox', 'daq_utils.py', '.rst'],
-                                         replace=False, replace_str="'limits'")
+    # paths = recursive_find_expr_in_files('C:\\Users\\weber\\Labo\\Programmes Python\\PyMoDAQ_Git',
+    #                                      exp='https://github.com/CEMES-CNRS',
+    #                                      paths=[],
+    #                                      filters=['.git', '.idea', '__pycache__', 'build', 'egg',
+    #                                               '.tox', 'daq_utils.py'],
+    #                                      replace=False, replace_str="https://github.com/PyMoDAQ")
     pass
     # paths = recursive_find_files('C:\\Users\\weber\\Labo\\Programmes Python\\PyMoDAQ_Git',
     #                      exp='VERSION', paths=[])
